%==========================================================================

% Code for the replication of the paper "Unconventional Monetary Policy, 
% Fiscal Side Effects and Euro Area (Im)balances", Journal of the European
% Economic Association, by Michael Hachula, Michele Piffer and Malte Rieth

% Michele Piffer, m.b.piffer@gmail.com
% https://sites.google.com/site/michelepiffereconomics/
% October 2018, Queen Mary, University of London

%==========================================================================


close all

do_importevents         = 0;  
do_importdata_variables = 0;  
windsor_shocks_daiy     = 0;


%% Import data on dummy variables for events

if do_importevents == 1 
    
    % %%% Organize time axis
    % trading days available from 1.1.1999 to 30.12.2016
    Step                 = xlsread('..\Data\ECB_daily_announcement_dummies.xlsx', 'Events', 'B3:D4698');
    assert( size(Step,1) == 4696 )
    assert( size(Step,2) == 3 )
    first_obs = 1;
    last_obs  = size(Step,1);
    year_shocks_daily  = Step(first_obs:last_obs,1);
    month_shocks_daily = Step(first_obs:last_obs,2);
    day_shocks_daily   = Step(first_obs:last_obs,3);
    T                  = length(year_shocks_daily);
    
    timeaxis_shocks_daiy = NaN*ones(T,1);
    step1 = unique(year_shocks_daily);
    for i = 1:length(step1)
        step1(i);
        step2 = year_shocks_daily == step1(i);
        step3 = find(step2);
        timeaxis_shocks_daiy(min(step3):min(step3)+length(step3)-1) = [step1(i):1/length(step3) :step1(i)+1-1/length(step3)]';
    end

    
    % %%% Organize event names
    [~,~,Events_names] = xlsread('..\Data\ECB_daily_announcement_dummies.xlsx', 'Events', 'E3:E4698'); 

    % %%% Organize classification of events
    excelclassif_conventional           = xlsread('..\Data\ECB_daily_announcement_dummies.xlsx', 'Events', 'G3:G4698');
    excelclassif_unconventional         = xlsread('..\Data\ECB_daily_announcement_dummies.xlsx', 'Events', 'K3:K4698');
    assert( sum(excelclassif_conventional)   == 220)
    assert( sum(excelclassif_unconventional) == 51)

    % %%% Classifications for the events 
    Dummies       = [excelclassif_conventional, ...
                     excelclassif_unconventional];
    Dummies_names = char('Conventional', ...
                         'Unconventional');
       
    % %%% Reorganize and save
    Dummies_names                = cellstr(Dummies_names);
    assert(size(Dummies_names,1) == size(Dummies,2))
    assert(size(Dummies,1)       == T )
    assert( max(max(Dummies))    == 1) 
    
    save('..\Temporary\Temporary_announcement_data', ...
                         'first_obs', 'last_obs', 'year_shocks_daily', 'month_shocks_daily', ...
                         'day_shocks_daily', 'timeaxis_shocks_daiy', 'T', ...
                         'Dummies', 'Dummies_names', 'Events_names');     
         
else
     load('..\Temporary\Temporary_announcement_data') ;
end


%% Import data for Bloomberg economic surprises, used as controls in the panel


if do_importdata_variables == 1 
    Step = xlsread('..\Data\Daily_data_Surprises.xls','tradingDaysLONG','B3:FQ4698'); 
    
    Step = Step(first_obs:last_obs,:); % news enter with the same timeaxis as the dummies
    
    assert(size(Step,1) == T )
    
     News = Step(:,[1:169]) ;
    correl = tril(corrcoef(News),-1) ;
    correl(correl > 0.85 & correl <= 1) = 10000 ;
    high_correl = sum(correl) ;
    high_correl(high_correl < 9000 ) = 1 ;
    high_correl(high_correl > 9000 ) = 0 ;
    index1 = find(high_correl) ;
    News = News(:,index1) ;
    
    News_obs = 0*News ;
    News_obs(News ~= 0) = 1;
    num_obs_per_series = sum(News_obs,1) ;
    num_obs_per_series(num_obs_per_series > 20 ) = 1 ;
    num_obs_per_series(num_obs_per_series > 1 ) = 0 ;
    index2 = find(num_obs_per_series) ;
    News = News(:,index2) ;
    assert(size(News,1) == length(year_shocks_daily) )
    
    save('..\Temporary\Temporary_surprise_data',...
             'News') ;    
else
    load('..\Temporary\Temporary_surprise_data') ;
end


%% Import data available for different countries (yields, spreads vs DE, ...)

if do_importdata_variables == 1 
    
    % Import data on yields
    Step = xlsread('..\Data\Data_daily_Yields.xlsx', 'Rearranged', 'B2:CC4697'); % has the same time axis as the dummies. 
    Step = Step(first_obs:last_obs,:); 
    assert( size(Step,1) == T )
       
    n_countries   = 20;
    n_maturities  = 4;
    assert( size(Step,2) == n_countries*n_maturities )
    
    % Organize data on yields
    yields_matrix                 = Step(:,1:n_countries*n_maturities);
    [~,~,yields_matrix_fullnames_step]     = xlsread('..\Data\Data_daily_Yields.xlsx','Legend', 'C2:C81'); 
    [~,~,yields_matrix_betternames_step]   = xlsread('..\Data\Data_daily_Yields.xlsx','Legend', 'D2:D81'); 
    yields_matrix_fullnames   = cell(n_countries*n_maturities,1);
    yields_matrix_betternames = cell(n_countries*n_maturities,1);
    string1 = ' yield';
    for i = 1:n_countries*n_maturities
        yields_matrix_fullnames(i)   = strcat(yields_matrix_fullnames_step(i),  string1);
        yields_matrix_betternames(i) = strcat(yields_matrix_betternames_step(i));
    end
    [~,~,yields_matrix_region]    = xlsread('..\Data\Data_daily_Yields.xlsx','Legend', 'E2:E81'); 
    yields_matrix_transformIRFs   = xlsread('..\Data\Data_daily_Yields.xlsx','Legend', 'G2:G81');
    [~,~,yields_matrix_labelIRFs] = xlsread('..\Data\Data_daily_Yields.xlsx','Legend', 'H2:H81'); 
    
    assert( size(yields_matrix_fullnames,1)     ==  n_countries*n_maturities) 
    assert( size(yields_matrix_betternames,1)   ==  n_countries*n_maturities) 
    assert( size(yields_matrix_transformIRFs,1) ==  n_countries*n_maturities) 
    assert( size(yields_matrix_labelIRFs,1)     ==  n_countries*n_maturities) 
      
    
    % Compute corresponding spreads, versus Germany, for each maturity
    Germany_position = find(strcmpi (yields_matrix_region, 'Germany'));
    assert( length(Germany_position) == 4 )   
    spreads_matrix = yields_matrix - kron( ones(1,n_countries), yields_matrix(:,Germany_position) );
    assert( max(max(abs(spreads_matrix(:,Germany_position)))) == 0 ) % spread of Germany on Germany should must be zero
    spreads_matrix(:,Germany_position) = NaN*ones(T,4);              % set this spread to NaN
    
    spreads_matrix_fullnames   = cell(n_countries*n_maturities,1);
    spreads_matrix_betternames = cell(n_countries*n_maturities,1);
    string1 = ' spread';
    for i = 1:n_countries*n_maturities
        spreads_matrix_fullnames(i)   = strcat(yields_matrix_fullnames_step(i),  string1);
        spreads_matrix_betternames(i) = strcat(yields_matrix_betternames_step(i));
    end
    spreads_matrix_region        = yields_matrix_region;
    spreads_matrix_transformIRFs = yields_matrix_transformIRFs;
    spreads_matrix_labelIRFs     = yields_matrix_labelIRFs;
           
    save('..\Temporary\Temporary_forcountries_data', ...
             'n_countries', 'n_maturities', ...
             'yields_matrix',  'yields_matrix_fullnames',  'yields_matrix_betternames',  'yields_matrix_region',  'yields_matrix_transformIRFs',  'yields_matrix_labelIRFs', ...
             'spreads_matrix', 'spreads_matrix_fullnames', 'spreads_matrix_betternames', 'spreads_matrix_region', 'spreads_matrix_transformIRFs', 'spreads_matrix_labelIRFs') ;
    
         
else
    load('..\Temporary\Temporary_forcountries_data') ;
end


%% Import data for variables not available for all countries 

if do_importdata_variables == 1 
    Step = xlsread('..\Data\Data_daily_Othervariables.xlsx','Rearranged','B2:CF4697'); % [change final number if add variables]
    
    Step = Step(first_obs:last_obs,:); 
    assert(size(Step,1) == T )
    
    othervariables = Step;

    [~,~,othervariables_fullnames]   = xlsread('..\Data\Data_daily_Othervariables.xlsx', 'Legend', 'C2:C84'); % [change last bit if add variables]
    [~,~,othervariables_betternames] = xlsread('..\Data\Data_daily_Othervariables.xlsx', 'Legend', 'D2:D84'); % [change last bit if add variables]
    [~,~,othervariables_region]      = xlsread('..\Data\Data_daily_Othervariables.xlsx', 'Legend', 'G2:G84'); % [change last bit if add variables]
    othervariables_transformIRFs     = xlsread('..\Data\Data_daily_Othervariables.xlsx', 'Legend', 'E2:E84'); % [change last bit if add variables]
    [~,~,othervariables_labelIRFs]   = xlsread('..\Data\Data_daily_Othervariables.xlsx', 'Legend', 'F2:F84'); % [change last bit if add variables]
    
    % convert some into logs
    step_convert_to_logs = find(strcmp(othervariables_fullnames, 'eurostoxx'));
    step_convert_to_logs = [step_convert_to_logs, find(strcmp(othervariables_fullnames, 'ITA msci'))];
    step_convert_to_logs = [step_convert_to_logs, find(strcmp(othervariables_fullnames, 'SPA msci'))];
    step_convert_to_logs = [step_convert_to_logs, find(strcmp(othervariables_fullnames, 'GER msci'))];
    step_convert_to_logs = [step_convert_to_logs, find(strcmp(othervariables_fullnames, 'FRA msci'))];
    step_convert_to_logs = [step_convert_to_logs, find(strcmp(othervariables_fullnames, 'BEL msci'))];
    step_convert_to_logs = [step_convert_to_logs, find(strcmp(othervariables_fullnames, 'NLD msci'))];
    step_convert_to_logs = [step_convert_to_logs, find(strcmp(othervariables_fullnames, 'LIT msci'))];
    step_convert_to_logs = [step_convert_to_logs, find(strcmp(othervariables_fullnames, 'POR msci'))];
    step_convert_to_logs = [step_convert_to_logs, find(strcmp(othervariables_fullnames, 'SVN msci'))];
    step_convert_to_logs = [step_convert_to_logs, find(strcmp(othervariables_fullnames, 'GRE msci'))];
    step_convert_to_logs = [step_convert_to_logs, find(strcmp(othervariables_fullnames, 'IRE msci'))];
    step_convert_to_logs = [step_convert_to_logs, find(strcmp(othervariables_fullnames, 'FIN msci'))];
    step_convert_to_logs = [step_convert_to_logs, find(strcmp(othervariables_fullnames, 'AUT msci'))];
    assert( length(step_convert_to_logs) == 14 ) % this is the number of variables that must be transformed into logs
    string1 = ', log';
    for step1 = 1:length(step_convert_to_logs)   
        step2 = step_convert_to_logs(step1);

        othervariables(:,step2)             = log(othervariables(:,step2));   
        othervariables_fullnames(step2)     = strcat(othervariables_fullnames(step2), string1);
        othervariables_betternames(step2)   = strcat(othervariables_betternames(step2));
        othervariables_transformIRFs(step2) = 100;
        othervariables_labelIRFs(step2)     = cellstr(char('%'));
    end

    othervariables_fullnames                    = cellstr(othervariables_fullnames);
    assert(size(othervariables_fullnames,1)     == size(othervariables,2))
    assert(size(othervariables_transformIRFs,1) == size(othervariables,2))
    assert(size(othervariables_labelIRFs,1)     == size(othervariables,2))
    
    save('..\Temporary\Temporary_othervariables_data', 'othervariables', 'othervariables_fullnames', 'othervariables_betternames', 'othervariables_region', 'othervariables_transformIRFs', 'othervariables_labelIRFs') ;

else
     load('..\Temporary\Temporary_othervariables_data') ;        
end


%% Reorganize daily data into a single matrix

% yields for all countries
alldailydata               = yields_matrix;
alldailydata_fullnames     = yields_matrix_fullnames; 
alldailydata_betternames   = yields_matrix_betternames; 
alldailydata_region        = yields_matrix_region;
alldailydata_transformIRFs = yields_matrix_transformIRFs;
alldailydata_labelIRFs     = yields_matrix_labelIRFs;

% add spreads for all countries
alldailydata               = [alldailydata,               spreads_matrix];
alldailydata_fullnames     = [alldailydata_fullnames;     spreads_matrix_fullnames]; 
alldailydata_betternames   = [alldailydata_betternames;   spreads_matrix_betternames]; 
alldailydata_region        = [alldailydata_region;        spreads_matrix_region]; 
alldailydata_transformIRFs = [alldailydata_transformIRFs; spreads_matrix_transformIRFs]; 
alldailydata_labelIRFs     = [alldailydata_labelIRFs;     spreads_matrix_labelIRFs]; 
    
% add other variables, not available for all countries
alldailydata               = [alldailydata,               othervariables];
alldailydata_fullnames     = [alldailydata_fullnames;     othervariables_fullnames]; 
alldailydata_betternames   = [alldailydata_betternames;   othervariables_betternames]; 
alldailydata_region        = [alldailydata_region;        othervariables_region]; 
alldailydata_transformIRFs = [alldailydata_transformIRFs; othervariables_transformIRFs]; 
alldailydata_labelIRFs     = [alldailydata_labelIRFs;     othervariables_labelIRFs]; 


% add Euro 10y yield - Euribor
j1 = find(strcmp(alldailydata_fullnames, '10y eurobond EA10')); 
j2 = find(strcmp(alldailydata_fullnames, 'euribor')); 
alldailydata               = [alldailydata,               alldailydata(:,j1) - alldailydata(:,j2)];
alldailydata_fullnames     = [alldailydata_fullnames;     'EA10 (EA11 without Greece) 10y yield - Euribor']; 
alldailydata_betternames   = [alldailydata_betternames;   'Term spread (10y - 3m rate)']; 
alldailydata_region        = [alldailydata_region;        'EA10']; 
alldailydata_transformIRFs = [alldailydata_transformIRFs; 100]; 
alldailydata_labelIRFs     = [alldailydata_labelIRFs;     'basis points']; 

% add Euro 10y yield - Euribor
j1 = find(strcmp(alldailydata_fullnames, '10y eurobond EA11')); 
j2 = find(strcmp(alldailydata_fullnames, 'euribor'));
alldailydata               = [alldailydata,               alldailydata(:,j1) - alldailydata(:,j2)];
alldailydata_fullnames     = [alldailydata_fullnames;     'EA11 10y yield - Euribor']; 
alldailydata_betternames   = [alldailydata_betternames;   'Term spread (10y - 3m rate)']; 
alldailydata_region        = [alldailydata_region;        'EA11']; 
alldailydata_transformIRFs = [alldailydata_transformIRFs; 100]; 
alldailydata_labelIRFs     = [alldailydata_labelIRFs;     'basis points']; 

% add Euro 10y yield - Euribor
j1 = find(strcmp(alldailydata_fullnames, '10y eurobond EA18')); 
j2 = find(strcmp(alldailydata_fullnames, 'euribor')); 
alldailydata               = [alldailydata,               alldailydata(:,j1) - alldailydata(:,j2)];
alldailydata_fullnames     = [alldailydata_fullnames;     'EA18 (EA19 without Greece) 10y yield - Euribor']; 
alldailydata_betternames   = [alldailydata_betternames;   'Term spread (10y - 3m rate)']; 
alldailydata_region        = [alldailydata_region;        'EA18']; 
alldailydata_transformIRFs = [alldailydata_transformIRFs; 100]; 
alldailydata_labelIRFs     = [alldailydata_labelIRFs;     'basis points']; 

% add Euro 10y yield - Euribor
j1 = find(strcmp(alldailydata_fullnames, '10y eurobond EA19')); 
j2 = find(strcmp(alldailydata_fullnames, 'euribor')); 
alldailydata               = [alldailydata,               alldailydata(:,j1) - alldailydata(:,j2)];
alldailydata_fullnames     = [alldailydata_fullnames;     'EA19 10y yield - Euribor']; 
alldailydata_betternames   = [alldailydata_betternames;   'Term spread (10y - 3m rate)']; 
alldailydata_region        = [alldailydata_region;        'EA19']; 
alldailydata_transformIRFs = [alldailydata_transformIRFs; 100]; 
alldailydata_labelIRFs     = [alldailydata_labelIRFs;     'basis points']; 

% add BBB-AAA spread 2y
j1 = find(strcmp(alldailydata_fullnames, 'corp bonds BBB 2y')); % current baseline
j2 = find(strcmp(alldailydata_fullnames, 'corp bonds AAA 2y pseudo')); % new baseline
alldailydata               = [alldailydata,               alldailydata(:,j1) - alldailydata(:,j2)];
alldailydata_fullnames     = [alldailydata_fullnames;     'BBB-AAA spread 2y']; 
alldailydata_betternames   = [alldailydata_betternames;   'BBB-AAA spread']; 
alldailydata_region        = [alldailydata_region;        'EA']; 
alldailydata_transformIRFs = [alldailydata_transformIRFs; 100]; 
alldailydata_labelIRFs     = [alldailydata_labelIRFs;     'basis points']; 
                 
k = size(alldailydata,2);
assert( T == size(alldailydata,1) )
assert( k == size(alldailydata,2) )
assert( k == size(alldailydata_fullnames,1) )    
assert( k == size(alldailydata_betternames,1) )    
assert( k == size(alldailydata_region,1) )   
assert( k == size(alldailydata_transformIRFs,1) )    
assert( k == size(alldailydata_labelIRFs,1) )    
 

%% Use only subperiod of interest
disp(['-->> Data available for :  ', num2str(year_shocks_daily(1)) 'm' num2str(month_shocks_daily(1)) 'd' num2str(day_shocks_daily(1))...
                          ' until ', num2str(year_shocks_daily(end)) 'm' num2str(month_shocks_daily(end))  'd' num2str(day_shocks_daily(end)) ] );
                        
if which_phase_considered ==  1  % full conventional
    from  = find(year_shocks_daily  == 1999 & month_shocks_daily == 1  & day_shocks_daily == 1);  
    until = find(year_shocks_daily  == 2007 & month_shocks_daily == 07 & day_shocks_daily == 31); 

elseif which_phase_considered == 2 % ump phase 1
    from  = find(year_shocks_daily  == 2007 & month_shocks_daily == 8  & day_shocks_daily == 1); 
    until = find(year_shocks_daily  == 2014 & month_shocks_daily == 5 & day_shocks_daily == 30); 

elseif which_phase_considered == 3 % ump phase 2
    from  = find(year_shocks_daily  == 2014 & month_shocks_daily == 6  & day_shocks_daily == 2);   
    until = find(year_shocks_daily  == 2016 & month_shocks_daily == 12 & day_shocks_daily == 30); 

end

assert( length(from)  == 1 ) 
assert( length(until) == 1 ) 


disp(['-->> Subperiod used :      ', num2str(year_shocks_daily(from)) 'm' num2str(month_shocks_daily(from)) 'd' num2str(day_shocks_daily(from))...
                          ' until ', num2str(year_shocks_daily(until)) 'm' num2str(month_shocks_daily(until))  'd' num2str(day_shocks_daily(until)) ] );
                        
                        
T_sub = length(year_shocks_daily(from:until));    
  

%% Generate regressor as dummy variable for the events

if which_phase_considered == 1
    which_dummy = 1; 

elseif which_phase_considered == 2 || which_phase_considered == 3
    which_dummy = 2; 
end


disp(['==== Selection of events used as dummies:    ===='] );
Dummies_names(which_dummy)

dummy_used   = Dummies(:,which_dummy);

% drop any event in the period not used
sum(dummy_used(1:from-1) );
sum(dummy_used(until+1:end) );
dummy_used(1:from-1)    = zeros(from-1, 1);
dummy_used(until+1:end) = zeros(T-until,1);

n_events        = sum(dummy_used);
X_events        = zeros(T,n_events);
step_eventposit = find(dummy_used == 1); % gives the position in which events occurred
for p = 1:n_events
    X_events(step_eventposit(p),p) = 1;
end
disp(['-->> Number of events used :      ', num2str(n_events)] );
assert(n_events <= sum(Dummies(:,which_dummy)))                         
                      

X_events_sub = X_events(from:until,:);
assert( size(X_events_sub,1) == T_sub ) 

step_eventposit_allevents = find(Dummies(:,1) == 1);


%% Organize data for the regressions featuring the dummy variables for the events
p            = 1;

if which_phase_considered == 1 || which_phase_considered == 2
    control_news = 1;
elseif which_phase_considered == 3
    control_news = 0;       % multicollinearity problems in phase 2
end

if which_phase_considered == 1
    
    %%% panel for conventional
    variable_lhs = 230; 

elseif which_phase_considered == 2 

    variable_lhs =        [1:3    5:7    25:27    38        46:47       49:51    53:54      57:59   67       69:71   ]; 

elseif which_phase_considered == 3
 
    variable_lhs = [ 9:11 13:15 29:31 73:75 77:79 ]; 

end

alldailydata_fullnames(variable_lhs)     

if length(variable_lhs) > 1
    panel_structure = 1;
elseif length(variable_lhs) == 1
    panel_structure = 0;    
end


% If plot variables for the entire sample period, irrespectively on the part used for the estimation    
% from_figure  = timeaxis_shocks_daiy(1); 
% until_figure = timeaxis_shocks_daiy(end);

% If plot variables only for the sample used for the estimation    
from_figure  = timeaxis_shocks_daiy(from);
until_figure = timeaxis_shocks_daiy(until);

figure(9899)
for i = 1:length(variable_lhs)+1
    
    if i <= length(variable_lhs)
        subplot(4, ceil(length(variable_lhs)/4), i)
        plot(timeaxis_shocks_daiy, alldailydata(:,variable_lhs(i)))
        title(alldailydata_fullnames(variable_lhs(i)))
        set(gca,'box','off')
        grid
        axis([from_figure until_figure -Inf Inf])
    end
end
set(gcf,'Position',[0 0 1350*.7 1000*.7])
movegui('northwest')  

summary_daily_timing = [year_shocks_daily, month_shocks_daily, day_shocks_daily];
summary_daily_all    = [summary_daily_timing, Dummies(:,1), dummy_used, alldailydata(:,variable_lhs)];


if panel_structure == 1

    disp(['==== Panel structure: variables used as LHS are   ===='] );
    alldailydata_fullnames(variable_lhs)    
    
    step_dataforpanel       = alldailydata(from:until,variable_lhs);
    step_countryFE_repeat   = alldailydata_region(variable_lhs);
    step_countryFE_norepeat = unique(alldailydata_region(variable_lhs));
        
    yyy_full = NaN*ones(T_sub*length(variable_lhs),1);
    xxx_full = NaN*ones(T_sub*length(variable_lhs),p + length(step_countryFE_norepeat));

    for i = 1:length(variable_lhs)   
        yyy_step = step_dataforpanel(:,i);

        xxx_step = NaN*ones(T_sub,p);
        for ppp = 1:p
            xxx_step(:,ppp) = [NaN*ones(ppp,1); yyy_step(1:end-ppp)];
        end

        % add country level fixed effects, one for each country, and no constant term 
        step1 = zeros(T_sub,length(step_countryFE_norepeat));
        j = find(strcmpi (step_countryFE_norepeat, step_countryFE_repeat(i)));
        step1(:,j) = ones(T_sub,1);
        xxx_step = [xxx_step, step1];

        yyy_full( (i-1)*T_sub + 1: (i-1)*T_sub + T_sub)   = yyy_step;
        xxx_full( (i-1)*T_sub + 1: (i-1)*T_sub + T_sub,:) = xxx_step;  
    end

   if control_news == 1
        % make sure the news controlled for have some nonzero values in the subperiod used
        News_step1 = News(from:until,:);
        News_step2 = News_step1==0;
        News_step3 = sum(News_step2,1);
        News_step4 = News_step3==T_sub;
        News_use   = News(from:until,News_step4==0);
       
        xxx_final = [ kron(ones(length(variable_lhs),1), X_events_sub), ...
                      kron(ones(length(variable_lhs),1), News_use),...
                      xxx_full];
    else
        xxx_final = [ kron(ones(length(variable_lhs),1), X_events_sub), ...
                      xxx_full];
    end
 
    % drop NaNs
    cond      = isnan(xxx_final);
    cond2     = sum(cond,2);
    sum(cond2);
    yyy_full  = yyy_full(cond2==0);
    xxx_final = xxx_final(cond2==0,:);

    % check that none of the observations dropped corresponded to any event considered
    step3 = find(cond2==1);
    step3 = from-1 + step3; % adjust for the fact that we might be using a subperiod
    assert( length([step_eventposit; step3]) == length(unique([step_eventposit; step3])) )

elseif panel_structure == 0

     disp(['==== No panel structure: variable used as LHS is   ===='] );
     alldailydata_fullnames(variable_lhs)    
    
     yyy_step = alldailydata(from:until,variable_lhs);
     assert( size(yyy_step,1) == T_sub )
     assert( size(yyy_step,2) == 1 )
         
     xxx_step = NaN*ones(T_sub,p);
     for ppp = 1:p
        xxx_step(:,ppp) = [NaN*ones(ppp,1); yyy_step(1:end-ppp)];
     end

     % add constant  
     xxx_step = [xxx_step, ones(T_sub,1)];

     yyy_full = yyy_step;
     xxx_full = xxx_step;  

    if control_news == 1
        % make sure the news controlled for have some nonzero values in the subperiod used
        News_step1 = News(from:until,:);
        News_step2 = News_step1==0;
        News_step3 = sum(News_step2,1);
        News_step4 = News_step3==T_sub;
        News_use   = News(from:until,News_step4==0);
        
        xxx_final = [X_events_sub, News_use, xxx_full];
        
    else
        xxx_final = [X_events_sub, xxx_full];
    end

    
    % drop NaNs
    cond      = isnan(xxx_final);
    cond2     = sum(cond,2);
    sum(cond2);
    yyy_full  = yyy_full(cond2==0);
    xxx_final = xxx_final(cond2==0,:);

    % check that none of the observations dropped corresponded to any event considered
    step3 = find(cond2==1);
    step3 = from-1 + step3;
    assert( length([step_eventposit; step3]) == length(unique([step_eventposit; step3])) )
    
end


%% Estimate parameters on the dummy variables used and convert into a daily proxy

disp(['==== Observations used and number of regressors:  ', num2str(size(xxx_final)) ' ===='] );

coeff = (xxx_final'*xxx_final)^(-1)*xxx_final'*yyy_full;

% check that there is no singularity problem that implies no available estimates
step1 = isnan(coeff) ;
step2 = sum(step1) ;
if step2 > 0
   display(['===  !!!PROBLEM!!! there must be a singularity problem in the variables chosen or control variables added as surprise ==='])
   assert(step2==0) 
end
coeff_events     = coeff(1:n_events); 
coeff_events_raw = coeff_events;

% Winsorize the instruments, if there are clear outliers 
if windsor_shocks_daiy == 1
    interval_win = 10;
    step_L       =  prctile(coeff_events, interval_win);
    step_H       =  prctile(coeff_events, 100-interval_win);
    stepw1       = coeff_events <= step_L;
    coeff_events(stepw1) = step_L;
    stepw2       = coeff_events >= step_H;
    coeff_events(stepw2) = step_H;
else
end

% Map estimated coefficients into daily shocks
shocks_daily = zeros(T,1);
for nnn = 1:n_events
   shocks_daily(step_eventposit(nnn)) = coeff_events(nnn); 
end

% Compute comulative shocks used for the Gertler and Karadi aggregation
shocks_daily_cumulative = zeros(T,1);
for t = 1:T
    shocks_daily_cumulative(t) = sum(shocks_daily(1:t));
end

% Set equal to NaN for the period not used in the estimation
assert( sum(shocks_daily(1:from-1) ) == 0 )
assert( sum(shocks_daily(until+1:end) ) == 0 )
shocks_daily(1:from-1)    = NaN*ones(from-1, 1);
shocks_daily(until+1:end) = NaN*ones(T-until,1);


assert( sum(shocks_daily_cumulative(1:from-1) ) == 0 )
shocks_daily_cumulative(1:from-1) = NaN*ones(from-1, 1);
shocks_daily_cumulative(until+1:end) = NaN*ones(T-until,1);

summary_daily = [year_shocks_daily, month_shocks_daily, day_shocks_daily, dummy_used, shocks_daily, shocks_daily_cumulative];


%% Compute monthly shocks

T_monthly               = (max(year_shocks_daily) - min(year_shocks_daily)+1)*12; 
year_shocks_monthly     = kron((min(year_shocks_daily):1: max(year_shocks_daily))',ones(12,1));
month_shocks_monthly    = kron(ones(max(year_shocks_daily) - min(year_shocks_daily)+1,1),(1:1:12)');
timeaxis_shocks_monthly = [year_shocks_monthly(1) : 1/12 : year_shocks_monthly(end) + 11/12];

assert(length(year_shocks_monthly)     == T_monthly);
assert(length(month_shocks_monthly)    == T_monthly);
assert(length(timeaxis_shocks_monthly) == T_monthly)


shocks_monthly = NaN*ones(T_monthly,1);   

for yyy = min(year_shocks_monthly):max(year_shocks_monthly)
    for mmm = min(month_shocks_monthly):max(month_shocks_monthly)
        % event(s) for this year and month
        cond_time = year_shocks_daily == yyy & month_shocks_daily== mmm;
        where = find(year_shocks_monthly == yyy & month_shocks_monthly == mmm);
        
        % Unconditional sum
        shocks_monthly(where,1) = sum(shocks_daily(cond_time));
    end
end


    figure(14)  
    subplot(2,1,1)
plot(timeaxis_shocks_daiy, shocks_daily)
set(gca,'box','off')
axis([min(timeaxis_shocks_daiy) max(timeaxis_shocks_daiy) -Inf Inf])
grid
title('Daily proxy')
    subplot(2,1,2)
plot(timeaxis_shocks_monthly, shocks_monthly), hold on
set(gca,'box','off')
axis([min(timeaxis_shocks_monthly) max(timeaxis_shocks_monthly) -Inf Inf])
grid
title('Monthly proxy')
set(gcf,'Position',[0 0 1000*.7 750*.7])
movegui('northeast')
set(gcf, 'PaperPositionMode', 'auto');

summary_shocks_monthly = [year_shocks_monthly, month_shocks_monthly, shocks_monthly];
summary_shocks_daily   = [year_shocks_daily, month_shocks_daily, day_shocks_daily, shocks_daily];


%% Compute dependent variable for RSW approach to daily beta, using the subperiod employed in the identification
alldailydata_diffaroundevents = zeros(size(alldailydata));
alldailydata_diffaroundevents(dummy_used==1,:) = alldailydata(step_eventposit,:) - alldailydata(step_eventposit-1,:); % 1 day window
 
% set it to NaN if variable was NaN
alldailydata_diffaroundevents(isnan(alldailydata)) = NaN;

% set it to NaN if that period was not used for the identification
alldailydata_diffaroundevents(1:from-1,:)    = NaN;
alldailydata_diffaroundevents(until+1:end,:) = NaN;


k = size(alldailydata,2);
assert( T == size(alldailydata,1) )
assert( k == size(alldailydata,2) )
assert( k == size(alldailydata_fullnames,1) )    
assert( k == size(alldailydata_betternames,1) )    
assert( k == size(alldailydata_region,1) )   
assert( k == size(alldailydata_transformIRFs,1) )    
assert( k == size(alldailydata_labelIRFs,1) )  
assert( k == size(alldailydata_diffaroundevents,2) )


%% SAVE data on shocks and on daily betas
save('..\Temporary\Temporary_ConstructedProxy_withdailybetas', ...
                             'year_shocks_daily', 'month_shocks_daily', 'day_shocks_daily', 'shocks_daily', 'summary_daily', ...
                             'year_shocks_monthly', 'month_shocks_monthly', 'timeaxis_shocks_monthly', 'shocks_monthly', 'summary_shocks_monthly', ...
                             'which_dummy', 'Dummies_names', 'n_events', 'variable_lhs', 'panel_structure', 'from', 'until', 'windsor_shocks_daiy', ...
                             'alldailydata', 'alldailydata_fullnames', 'alldailydata_betternames', 'alldailydata_region', 'alldailydata_transformIRFs', 'alldailydata_labelIRFs', ...
                             'alldailydata_diffaroundevents', ...
                             'which_phase_considered')
                                                  
'instrument saved'    

pause(1)


close all
